/*
  Debounce.cpp - a library for debouncing switch inputs
  Paul MacDougal
  May 2015

  This work is licensed under the Creative Commons Attribution 4.0 International License.
  To view a copy of this license, visit http://creativecommons.org/licenses/by/4.0/ or send a letter to Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.
*/

#include <Arduino.h>
#include "DeBounce.h"

#define STATE_UNKNOWN  0
#define STATE_IDLE     1
#define STATE_GREEN    2
#define STATE_RED      3

#if defined(__AVR_ATtiny84__) || defined(__AVR_ATtiny85__)
#define USE_SERIAL 0
#else
#define USE_SERIAL 1
#endif

// set this to 1 to get some Serial output on non ATtiny parts
#define VERBOSE 0

#if VERBOSE && USE_SERIAL
    #define PRINT(a) Serial.print(a)
    #define PRINTLN(a) Serial.println(a)
#else
    #define PRINT(a)
    #define PRINTLN(a)
#endif

DeBounce::DeBounce()
{
    m_sw_pin = 0;
    m_red_level = LOW;
    m_state = STATE_UNKNOWN;
    m_startTime = 0;
    m_lastFalling = 0;
    m_settleTime = 0;
}

DeBounce::~DeBounce()
{
}

unsigned char DeBounce::init(unsigned char pin, unsigned char redLevel, unsigned long settleTime)
{
    // range check pin value here
#if defined(__AVR_ATtiny85__)
    if (pin > 4) {
        PRINTLN("Pin is out of range");
        return 1;
    }
#elif defined(__AVR_ATtiny84__)
    if (pin > 10) {
        PRINTLN("Pin is out of range");
        return 1;
    }
#elif defined(__AVR_ATmega328P__)
    if (pin > 13) {
        PRINTLN("Pin is out of range");
        return 1;
    }
#elif defined(__SAM3X8E__) // DigiX
    if (pin > 113) {
        PRINTLN("Pin is out of range");
        return 1;
    }
// #else
// #error What chip are you using?  Write more code
#endif

    m_sw_pin = pin;
    m_red_level = redLevel;
    m_settleTime = settleTime;
    pinMode(m_sw_pin, INPUT);

    // See if switch is UP or DOWN
    if (digitalRead(m_sw_pin) == m_red_level) {
        m_state = STATE_IDLE;
        m_startTime = m_lastFalling = micros();
    } else {
        m_state = STATE_GREEN;
        m_startTime = m_lastFalling = 0;
    }

    return 0;
}

unsigned char DeBounce::handleInput()
{
    unsigned char posn = digitalRead(m_sw_pin);

    // The most common case is the switch is up
    if (STATE_IDLE == m_state && m_red_level == posn) {
        return DeBounce_UP;
    }

    // next most common is the switch is being held down
    if (STATE_GREEN == m_state && m_red_level != posn) {
        return DeBounce_DOWN;
    }


    // next is waiting for the switch release bouncing to settle
    if (STATE_RED == m_state && m_red_level == posn) {
        unsigned long duration = micros() - m_lastFalling;
        if (duration > m_settleTime) {
            // This is what we have been waiting for!!!
            PRINT("Button press detected ");
            PRINTLN(duration);
            m_state = STATE_IDLE;
            // You can measure the duration of the press by m_lastFalling - m_startTime;
            return DeBounce_PRESS;
        } else {
            // still waiting for it to settle
            return DeBounce_UP;
        }
    }

    // Now handle the unusual cases, the transitions

    // _______|--------- (rising edge)
    if (STATE_RED == m_state && m_red_level != posn) {
        PRINTLN("Rising edge detected");
        m_state = STATE_GREEN;
        return DeBounce_RISING;
    }

    // _______|--------- (rising edge)
    if (STATE_IDLE == m_state && m_red_level != posn) {
        PRINTLN("Rising edge detected");
        m_startTime = micros();
        m_state = STATE_GREEN;
        return DeBounce_FIRST_RISING;
    }

    // -------|_________ (falling edge)
    // could be bounce or release of switch
    if (STATE_GREEN == m_state && m_red_level == posn) {
        PRINTLN("Falling edge detected");
        m_lastFalling = micros();
        m_state = STATE_RED;
        return DeBounce_FALLING;
    }

    return DeBounce_THIS_SHOULD_NEVER_HAPPEN;
}

unsigned long DeBounce::getDuration()
{
    // This includes the time it takes the switch to bounce on release
    if (STATE_IDLE == m_state)
        return (m_lastFalling -  m_startTime);
    else
        return(micros() - m_startTime);
}


